using System;
using System.Collections;
using System.Text;

namespace at.jku.ssw.Analyse {
	/// <summary>
	/// Dies ist die Oberklasse fr alle Info-Klassen. Hier werden alle allgemein
	/// gltigen Felder und Methoden definiert.
	/// </summary>
	public class CodeInfo {
		public String sName;	//Name des Objekts
		public int iLine;		//Zeile der Definition
		protected CodeInfo parent;	//Vater-Objekt
		public ArrayList children;	//Liste der Kinder-Objekte
		public ArrayList declaredVariables;	//Liste aller deklarierten Variablen
		protected bool hasComputed = false;		//Wurde compute schon aufgerufen
		public LinesOfCodeInfo locInfo;		//LinesOfCode Infos
		public StatementCountInfo statInfo;	//Statments Infos
		public HalsteadInfo hInfo;	//Infos ber das Komplexittsmass nach Halstead

		public CodeInfo(String sName, int iLine, CodeInfo parent) {
			this.sName = sName;
			this.iLine = iLine;
			this.parent = parent;
			if (parent != null)
				parent.addChild(this);
			children = new ArrayList();
			declaredVariables = new ArrayList();
			locInfo = new LinesOfCodeInfo();
			statInfo = new StatementCountInfo();
			hInfo = new HalsteadInfo();
		}

		/// <summary>
		/// Gibt den vollen Namen eines Objektes zurck. Dieser ist
		/// "Namespacename"."Klassenname"."Methodenname"("Parameteranzahl").
		/// Dabei mssen nicht immer alle Informationen gegeben sein.
		/// </summary>
		/// <returns>Den vollen Namen des Objekts</returns>
		public virtual String getFullName() {
			if (parent != null) {
				String sTmp = parent.getFullName();
				return (sTmp == "") ? sName : sTmp + "." + sName;
			}
			else return sName;
		}

		/// <summary>
		/// Die getCntChildren-Methode gibt die Anzahl der Kinder-Objekte
		/// des aktuellen Objekts zurck.
		/// </summary>
		/// <returns>Die Anzahl der Kinder-Objekte</returns>
		public int getCntChildren() {
			return children.Count;
		}

		/// <summary>
		/// Diese Methode fgt ein neues Kind-Objekt zu dem aktuellen Objekt hinzu.
		/// </summary>
		/// <param name="child">Das hinzuzufgende Objekt</param>
		public void addChild(CodeInfo child) {
			children.Add(child);
		}

		/// <returns>Den Vater des aktuellen Objekts</returns>
		public CodeInfo getParent() {
			return parent;
		}

		/// <summary>
		/// Zhlt die Anzahl der NamespaceInfo-Objekte im Informationsbaum
		/// </summary>
		public int getCntNamespaces() {
			int ret = 0;
			foreach (CodeInfo ci in children) {
				if (ci.GetType() == typeof(NamespaceInfo)) ret++;
				ret += ci.getCntNamespaces();
			}
			return ret;
		}

		/// <summary>
		/// Zhlt die Anzahl der ClassInfo-Objekte im Informationsbaum
		/// </summary>
		public int getCntClasses() {
			int ret = 0;
			foreach (CodeInfo ci in children) {
				if (ci.GetType() == typeof(ClassInfo)) ret++;
				ret += ci.getCntClasses();
			}
			return ret;
		}

		/// <summary>
		/// Zhlt die Anzahl der Variablen die in Klassen definiert wurden
		/// --> Felder
		/// </summary>
		public int getCntFields() {
			int ret = 0;
			foreach (CodeInfo ci in children) {
				if (ci.GetType() == typeof(ClassInfo)) ret += ci.getCntVariables();
				ret += ci.getCntFields();
			}
			return ret;
		}


		/// <returns>Die Anzahl der deklarierten Variablen in diesem Bereich. 
		/// Fr Klassen, die Klassenvariablen, fr Methoden die lokalen Variablen.</returns>
		public int getCntVariables() {
			return declaredVariables.Count;
		}

		/// <summary>
		/// Fgt eine neue Variable zum aktuellen Objekt hinzu. Aufgerufen fr Klassen 
		/// (Klassenvariablen) und Methoden (lokale Variablen).
		/// </summary>
		public void addVariable(VariableInfo variable) {
			declaredVariables.Add(variable);
		}

		/// <summary>
		/// Diese Methode steigt im Informationsbaum bis zur Wurzel auf, und gibt
		/// diese zurck.
		/// </summary>
		/// <returns>Den Wurzel-Knoten des Informationsbaums</returns>
		public CodeInfo getRoot() {
			if (parent != null)
				return parent.getRoot();
			return this;
		}

		/// <summary>
		/// Diese Methode sucht die Variable mit dem vollen Namen sName
		/// im Baum der ermittelten Informationen
		/// </summary>
		/// <param name="sName">Der volle Name einer Variable</param>
		/// <returns>Ein VariableInfo-Objekt falls gefunden, null sonst</returns>
		public VariableInfo findVariable(String sName) {
			VariableInfo viInfo = null;
			foreach(VariableInfo vi in declaredVariables)
				if (vi.getFullName() == sName) return vi;
			foreach(CodeInfo ci in children) {
				if ((viInfo = ci.findVariable(sName)) != null)
					return viInfo;
			}
			return null;
		}

		/// <summary>
		/// Diese Methode sucht die Methode mit dem vollen Namen sName
		/// im Baum der ermittelten Informationen
		/// </summary>
		/// <param name="sName">Der volle Name einer Methode</param>
		/// <returns>Ein MethodInfo-Objekt falls gefunden, null sonst</returns>
		public virtual MethodInfo findMethod(String sName) {
			MethodInfo methInfo = null;
			foreach(CodeInfo ci in children) {
				if ((methInfo = ci.findMethod(sName)) != null)
					return methInfo;
			}
			return null;
		}

		/// <summary>
		/// Diese Methode sucht die Klasse mit den Namen sName, und versucht
		/// daraus einene vollen Namen zu konstruieren, der dann gesucht wird.
		/// </summary>
		/// <param name="sName">der Name der Klasse</param>
		/// <returns>Ein ClassInfo-Objekt falls gefunden, null sonst</returns>
		public ClassInfo findClassNFN(String sName) {
			// es werden alle bergeordneten namespaces und und die globalen usings
			// gesucht, um die Klasse zu finden
			CodeInfo ciTmp = parent;
			UsingInfo uiTmp = null;
			while (ciTmp != null) {
				if (ciTmp.GetType() == typeof(RootInfo))
					uiTmp = ((RootInfo)ciTmp).uiUsings;
				else if (ciTmp.GetType() == typeof(NamespaceInfo))
					uiTmp = ((NamespaceInfo)ciTmp).uiUsings;
				else uiTmp = null;
				// Falls man hier ein UsingInfo Objekt gefunden hat, dann
				// wird mit diesem versucht, den vollen Namen der Klasse zu konstruieren
				// und dann zu finden
				if (uiTmp != null) {
					foreach (String s in uiTmp.getPossibilities(sName, getFullName())) {
						ClassInfo clsTmp;
						if ((clsTmp=getRoot().findClass(s)) != null) {
							// gefundene Klasse zurckgeben
							return clsTmp;
						}
					}
				}
				ciTmp = ciTmp.getParent();
			}
			return null;
		}

		/// <summary>
		/// Diese Methode sucht die Klasse mit dem vollen Namen sName
		/// im Baum der ermittelten Informationen
		/// </summary>
		/// <param name="sName">Der volle Name einer Klasse</param>
		/// <returns>Ein ClassInfo-Objekt falls gefunden, null sonst</returns>
		public virtual ClassInfo findClass(String sName) {
			ClassInfo clsInfo = null;
			foreach(CodeInfo ci in children) {
				if ((clsInfo = ci.findClass(sName)) != null)
					return clsInfo;
			}
			return null;
		}

		public virtual void compute() {
			if (hasComputed) return;
			hasComputed = true;
			foreach (CodeInfo ci in children){
				ci.compute();
				hInfo = ci.hInfo + hInfo;
			}
			hInfo.compute();
		}

		public override String ToString() {
			StringBuilder sb = new StringBuilder();
			sb.Append(ToString("", true));
			return sb.ToString();
		}

		protected StringBuilder ToDescend(String sTab) {
			StringBuilder sb = new StringBuilder();
			StringBuilder tab = new StringBuilder(sTab);
			tab.Append("  ");
			foreach (CodeInfo ci in children) sb.Append(ci.ToString(tab.ToString(), true));
			return sb;
		}

		public virtual String ToString(String sTab, bool bToDescend) {
			StringBuilder sb = new StringBuilder();
			StringBuilder tab = new StringBuilder(sTab);
			sb.Append(tab).AppendFormat("Name: {0} | Full-Name: {1}\n", sName, getFullName());
			sb.Append(locInfo.ToString(tab));
			sb.Append(statInfo.ToString(tab));
			sb.Append(hInfo.ToString(tab));
			sb.Append(sTab).Append("Variables: ").Append(getCntVariables()).Append("\n");
			foreach (VariableInfo vi in declaredVariables) {
				sb.Append(sTab).Append(" ").Append(vi.ToString()).Append("\n");
			}

			return sb.ToString();
		}
	}
}
